#!/bin/bash
#
# Last update: 2023.12.19

# 輸入 -h 或 --help 會顯示手動安裝說明
if [ "$1" == "-h" ] || [ "$1" == "--help" ]; then

clear
echo "===================================================================================================="
echo ""
echo "手動安裝 CXS 請執行 /opt/cpinst/cpinst/install_script/cxs.sh"
echo ""
echo "手動安裝 CSF 請執行 /opt/cpinst/cpinst/install_script/csf.sh"
echo ""
echo "手動安裝 Openresty(Nginx) 請執行 /opt/cpinst/cpinst/install_script/openresty.sh"
echo ""
echo "手動安裝 remoteip(Apache Module) 請執行 /opt/cpinst/cpinst/install_script/remoteip.sh"
echo ""
echo "手動安裝 CageFS(Cloudlinux) 請執行 /opt/cpinst/cpinst/install_script/cagefs.sh"
echo ""
echo "手動安裝 PHPSelector(Cloudlinux) 請執行 /opt/cpinst/cpinst/install_script/phpselector.sh"
echo ""
echo "手動安裝 Apache lsapi 請執行 /opt/cpinst/cpinst/install_script/lsapi.sh"
echo ""
#echo "手動安裝 Mysql Governor(Cloudlinux) 請執行 /opt/cpinst/cpinst/install_script/mysqlgovernor.sh"
#echo ""
echo "手動安裝 cPanel面板 翻譯 請執行 /opt/cpinst/cpinst/install_script/themes.sh"
echo ""
echo "手動安裝 Safe-rm 請執行 /opt/cpinst/cpinst/install_script/safe-rm.sh"
echo ""
echo "手動安裝 fcgi 請執行 /opt/cpinst/cpinst/install_script/fcgi.sh"
echo ""
echo "===================================================================================================="
exit

fi


# 如果系統為 CentOS 6 或 CentOS 7，會直接離開 Script
OS_version=$(cat /etc/redhat-release | grep 'release 6\|release 7' 2>/dev/null)
if [ "${OS_version}" ]; then
   echo "此主機系統為 CentOS 6 或 CentOS 7，無法正常使用此 Script 安裝 cPanel 環境。"
   exit
fi

# 如果系統已經安裝 cPanel，會直接離開 Script
if [ -e /usr/local/cpanel/cpanel ]; then
   echo "此主機已經安裝 cPanel，無法使用此 Script"
   exit
fi

# 設定 nameserver (只會詢問一次)
if [ ! -e /usr/local/src/set_nameserver ]; then

ans=''
while [ "$ans"=="1" -o "$ans"=="2" -o "$ans"=="3" ]; do
   clear
   echo "==================================================="
   echo "  請選擇主機存放位置，以便設定正確的 nameserver    "
   echo "                                                   "
   echo "  (1) 遠傳 (2) 宏遠 (3) 國外主機                   "
   echo "==================================================="
   read -e -p "您的選擇是 ? ==> (1/2/3): " -i "" ans
   case $ans in
        1)
           sed -i '/nameserver/d' /etc/resolv.conf
           echo "nameserver 103.17.9.155" >> /etc/resolv.conf
           echo "nameserver 103.17.11.155" >> /etc/resolv.conf
           echo "nameserver 168.95.1.1" >> /etc/resolv.conf
           touch /usr/local/src/set_nameserver
           break;;
        2)
           sed -i '/nameserver/d' /etc/resolv.conf
           echo "nameserver 168.95.1.1" >> /etc/resolv.conf
           echo "nameserver 8.8.8.8" >> /etc/resolv.conf
           touch /usr/local/src/set_nameserver
           break;;
        3)
           sed -i '/nameserver/d' /etc/resolv.conf
           echo "nameserver 8.8.8.8" >> /etc/resolv.conf
           echo "nameserver 4.2.2.1" >> /etc/resolv.conf
           touch /usr/local/src/set_nameserver
           break;;
        *)
           echo ""
           echo "輸入選擇為不正確 : $ans，請參考以下說明，輸入正確值"
           echo ""
           echo "(1) 遠傳 (2) 宏遠 (3) 國外主機"
           sleep 3
           continue;;
   esac
done

fi

# 系統是 CentOS，才會詢問是否要轉為 CloudLinuxOS (只會詢問一次)

CT=`grep CloudLinux /etc/redhat-release`

if [ ! -e /usr/local/src/no_install_cloudlinux ]; then

if [ ! "$CT" ]; then
  ans=''
  while [ "$ans"=="y" -o "$ans"=="n" ]; do
     clear
     echo "==========================================================="
     echo "  偵測到不是 CloudLinux OS，您是否要先轉為 CloudLinux OS ? "
     echo "  (轉換後會自動重開機，請重新執行此程序) $0                "
     echo "                                                           "
     echo "  CloudLinux 要先有授權才能安裝                            "
     echo "                                                           "
     echo "  (y) 轉換為 Cloudlinux OS (n) 不進行轉換                  "
     echo "==========================================================="
     read -e -p "您的選擇是 ? ==> (y/n): " -i "n" ans
     case $ans in
      y)
         yum install wget -y
         wget http://repo.cloudlinux.com/cloudlinux/sources/cln/cldeploy
         sh cldeploy -i
         rm -f cldeploy
         shutdown -rf now
         exit
         break;;
      n)
         installcl="no"
         touch /usr/local/src/no_install_cloudlinux
         break;;
      *)
         echo ""
         echo "輸入選擇為不正確 : $ans，請參考以下說明，輸入正確值"
         echo ""
         echo "(y) 轉換為 Cloudlinux OS (n) 不進行轉換"
         sleep 3
         continue;;
     esac
  done
fi

fi

# 將 "[root@cpXX ~]#" 改成 "root@cpXX [~]#" (只會設定一次)
# \u 使用者名稱 \h 主機名稱 \w 目前工作目錄
if [ ! -e /usr/local/src/set_ps1 ]; then
PS1="\u@\h [\w]# "
echo 'PS1="\u@\h [\w]# "' >> ~/.bashrc 
touch /usr/local/src/set_ps1
fi

# 設定 ls alias，讓 ll 指令可顯示隱藏檔案 (只會設定一次)
if [ ! -e /usr/local/src/set_ls_alias ]; then
alias ll='ls -l --color=tty -F -a -b -T 0'
cat << EOF >> /etc/bashrc
LS_OPTIONS='--color=tty -F -a -b -T 0';
export LS_OPTIONS;
EOF
alias ls='/bin/ls $LS_OPTIONS'
touch /usr/local/src/set_ls_alias
fi

# 設定時區區域為 Taipei (只會設定一次)
if [ ! -e /usr/local/src/set_timezone ]; then
/usr/bin/timedatectl set-timezone Asia/Taipei
touch /usr/local/src/set_timezone
fi

# fix csf CSF MESSENGER bug
systemctl stop nftables
systemctl disable nftables

# 紀錄開始安裝時間
echo "開始安裝 : `date`" > /root/install_date

# 關閉 KDUMP (只會設定一次)
if [ ! -e /usr/local/src/disable_kdump ]; then
/usr/bin/systemctl stop kdump.service
/usr/bin/systemctl disable kdump.service
touch /usr/local/src/disable_kdump
fi

# 註冊 CloudLinux Server (如果系統是 CentOS，會跳過註冊，只會執行一次)
if [ ! -e /usr/local/src/update_cloudlinux_license ]; then
if [ "$CT" ]; then
  /usr/sbin/clnreg_ks --force
  touch /usr/local/src/update_cloudlinux_license
fi
fi

# 更新系統套件並設定 Cron (只會設定一次)
if [ ! -e /usr/local/src/set_yum_cron ]; then

# CentOS8避免發生更新 grub2 套件，觸發無法正常開機進系統的 bug
# https://bugzilla.redhat.com/show_bug.cgi?id=1861977
#echo "exclude=grub2* shim* mokutil" >> /etc/yum.conf

/usr/bin/dnf upgrade -y
/usr/bin/echo "0 6 * * * root /usr/bin/dnf upgrade -y > /dev/null 2>&1" >> /etc/crontab
touch /usr/local/src/set_yum_cron
fi

# 關閉 SElinux (只會設定一次)
if [ ! -e /usr/local/src/disable_selinux ]; then
/usr/bin/sed -i 's/^SELINUX=.*/SELINUX=disabled/g' /etc/selinux/config
/usr/sbin/setenforce 0
touch /usr/local/src/disable_selinux
fi

# 安裝 chrony 時間對時，調整 chrony.conf 設定檔，設定 chrony 自動校時 (只會詢問一次)
if [ ! -e /usr/local/src/set_chrony ]; then
/usr/bin/yum install chrony -y
sed -i 's/pool 2.almalinux.pool.ntp.org iburst/#pool 2.almalinux.pool.ntp.org iburst/g' /etc/chrony.conf
sed -i 's/pool 2.cloudlinux.pool.ntp.org iburst/#pool 2.cloudlinux.pool.ntp.org iburst/g' /etc/chrony.conf
echo 'server 103.17.9.155 iburst' >> /etc/chrony.conf
echo 'server 10.100.9.155 iburst' >> /etc/chrony.conf
echo 'server 10.168.32.155 iburst' >> /etc/chrony.conf
#echo 'server tock.stdtime.gov.tw iburst' >> /etc/chrony.conf
systemctl start chronyd
systemctl enable chronyd
touch /usr/local/src/set_chrony 
fi

# 設定 SSH Config 修正 SSH 安全性、並重啟服務 (只會設定一次)
if [ ! -e /usr/local/src/update_sshd_conf ]; then
/usr/bin/echo "Ciphers aes128-ctr,aes192-ctr,aes256-ctr" >> /etc/ssh/sshd_config
/usr/bin/echo "MACs hmac-sha1" >> /etc/ssh/sshd_config
sed -i "s/#UseDNS no/UseDNS no/g" /etc/ssh/sshd_config
sed -i "s/#UseDNS yes/UseDNS no/g" /etc/ssh/sshd_config
/usr/bin/systemctl restart sshd.service
touch /usr/local/src/update_sshd_conf
fi

# 增加 sysctl.conf 設定 (只會設定一次)
if [ ! -e /usr/local/src/set_sysctl_conf ]; then
/usr/bin/cat <<EOF >> /etc/sysctl.conf
##################### Custom modify start #####################
 ## increase TCP max buffer size setable using setsockopt()
net.core.rmem_max = 16777216
net.core.wmem_max = 16777216
 ## increase Linux autotuning TCP buffer limits
 ## min, default, and max number of bytes to use
 ## set max to at least 4MB, or higher if you use very high BDP paths
net.ipv4.tcp_rmem = 4096 87380 16777216
net.ipv4.tcp_wmem = 4096 65536 16777216
 ## don't cache ssthresh from previous connection
net.ipv4.tcp_no_metrics_save = 1
net.ipv4.tcp_moderate_rcvbuf = 1
 ## recommended to increase this for 1000 BT or higher
net.core.netdev_max_backlog = 2500
 ## disable tcp selective acknowledgements.
net.ipv4.tcp_sack = 0
 ##enable window scaling
net.ipv4.tcp_window_scaling = 1
##################### Custom modify end #####################
EOF
/usr/sbin/sysctl -p
touch /usr/local/src/set_sysctl_conf
fi

# 關閉 firewalld、啟用 iptables (只會設定一次)
if [ ! -e /usr/local/src/disable_firewalld ]; then
/usr/bin/systemctl disable firewalld.service
/usr/bin/systemctl stop firewalld.service
/usr/bin/yum -y install iptables-services
/usr/bin/touch /etc/sysconfig/iptables
/usr/bin/touch /etc/sysconfig/iptables6
/usr/bin/systemctl start iptables
/usr/bin/systemctl start ip6tables
/usr/bin/systemctl enable iptables
/usr/bin/systemctl enable ip6tables
touch /usr/local/src/disable_firewalld
fi

# 安裝必要套件 (只會安裝一次)
if [ ! -e /usr/local/src/install_need_package ]; then
/bin/dnf -y install openldap-clients nss-pam-ldapd perl nc vim tar
/bin/dnf -y install traceroute net-tools elinks tcpdump mutt

# 預設almalinux8最小安裝並不會裝rsyslog，需要手動安裝
/bin/dnf install -y rsyslog
systemctl start rsyslog
systemctl enable rsyslog

# 系統日誌保留改成 12 week
sed -i 's/^rotate 4/rotate 12/g' /etc/logrotate.conf

# 系統日誌啟用壓縮
sed -i 's/^#compress/compress/g' /etc/logrotate.conf

touch /usr/local/src/install_need_package
fi

############################
### 顯示安裝 cPanel 選單 ###
############################

# 選擇主機使用者類型
ans=''
host_use_user=''
while [ "$ans"=="1" -o "$ans"=="2" ]; do
   clear
   echo "==================================================="
   echo "  請選擇 主機使用者 類型                           "
   echo "                                                   "
   echo "  (1) 遠振虛擬主機專用(客戶主機請勿選取)           "
   echo "  (2) 客戶主機專用(單次部屬&管理服務)              "
   echo "==================================================="
   read -e -p "您的選擇是 ? ==> (1/2): " -i "" ans
   case $ans in
        1)
           host_use_user="1"
           break;;
        2)
           host_use_user="2"
           break;;
        *)
           echo ""
           echo "輸入選擇為不正確 : $ans，請參考以下說明，輸入正確值"
           echo ""
           echo "(1) 自用虛擬主機 (2) 客戶主機"
           sleep 3
           continue;;
   esac
done

sleep 1
clear


# 定義各項目預設參數

# 1.外網網卡名稱
CPANEL_INTERFACE=`ip -o link show | awk -F': ' '{print $2}'|grep -v lo|head -n 1`

# 2.Main IP
#MAIN_IP=`ifconfig $CPANEL_INTERFACE| grep -Eo 'inet (addr:)?([0-9]*\.){3}[0-9]*' | grep -Eo '([0-9]*\.){3}[0-9]*' | grep -v '127.0.0.1'`
MAIN_IP=`ip route get 1.2.3.4 | grep via | awk '{print $7}'`

# 3.Main Share IP
MAIN_SHARE_IP="請手動輸入正確資訊"
if (("${host_use_user}" == "2")); then
   MAIN_SHARE_IP="$MAIN_IP"
fi

# 4.Hostname
SERVER_HOSTNAME=`hostname`

# 5.第一組 DNS 主機名稱
NS0="請手動輸入正確資訊"
if (("${host_use_user}" == "2")); then
   NS0=""
fi

# 6.第二組 DNS 主機名稱
NS1="請手動輸入正確資訊"
if (("${host_use_user}" == "2")); then
   NS1=""
fi

# 7.接收 System 通知郵件
CUSTOMER_EMAIL="cplog@ipmi.tw"
if (("${host_use_user}" == "2")); then
   CUSTOMER_EMAIL="請手動輸入正確資訊"
fi

# 8.接收 cPanel 通知郵件
WHM_EMAIL="whmlog@ipmi.tw"
if (("${host_use_user}" == "2")); then
   WHM_EMAIL="請手動輸入正確資訊"
fi

# 9.PHP版本(套用的 EA4 Json 設定檔)
eapver_version="82"

# 10.安裝 Openresty (Nginx)
#installnginx="yes"
#if (("${host_use_user}" == "2")); then
#   installnginx="no"
#fi

# 11.安裝 Default.htm 帳號預設頁面
installindex="yes"
if (("${host_use_user}" == "2")); then
   installindex="no"
fi

# 此套件因遭原廠檢舉，從2023.5.18發現無法繼續安裝，故註解取消使用
# 12.安裝 Let's Encrypt cPanel Plugin
#installletsencrypt="yes"
#if (("${host_use_user}" == "2")); then
#   installletsencrypt="no"
#fi

# 13.安裝本公司台灣或美國方案
installpackages="請手動輸入正確資訊"
if (("${host_use_user}" == "2")); then
   installpackages="no"
fi

# 14.安裝 CXS
installcxs="yes"
if (("${host_use_user}" == "2")); then
   installcxs="no"
fi

# 15.安裝 CageFS、PHP Selector、MySQL Governor (Cloudlinux)
installcagefs="yes"
if (("${host_use_user}" == "2")); then
   installcagefs="no"
fi

# 16.安裝 Installatron
installinstallatron="yes"
if (("${host_use_user}" == "2")); then
   installinstallatron="no"
fi

# 17.安裝 KernelCare
kernelupdaten="yes"
if (("${host_use_user}" == "2")); then
   kernelupdaten="no"
fi

# 18.安裝 Zabbix
zabbix="yes"
if (("${host_use_user}" == "2")); then
   zabbix="no"
fi

# 19.每小時寄信限制
mail_qty="300"

# 20.資料庫類型
mysql_version="8.0"

# 21.安裝 lsapi
lsapi="yes"

# 22.虛擬主機類型
virtualhost_type='php'

# 23.開放使用者 SSH 登入
allow_user_ssh='no'


# 依照選擇主機使用者類型，顯示不同確定清單

ans=''
while [ "$ans"=="y" -o "$ans"=="q" ]; do
   clear
   echo "========================================================================"
   echo "  以下為預設選項設定，請務必確認無誤，再進行下一步                      "
   #if [ "${virtualhost_type}" == "php" ]; then
   #   echo -e "  目前安裝的是 \e[0;32mPHP 虛擬主機\e[0m                          "
   #elif [ "${virtualhost_type}" == "python" ]; then
   #   echo -e "  目前安裝的是 \e[0;32mPython 虛擬主機\e[0m                       "
   #   # Python主機使用不到以下套件
   #   lsapi="no"
   #   installinstallatron="no"
   #else
   #   echo -e "\e[0;32m目前選擇的虛擬主機類型錯誤或是未設定\e[0m                 "
   #fi
   echo "                                                                        "
   
if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "1.外網網卡名稱 : ${CPANEL_INTERFACE}"
fi
if (("${host_use_user}" == "1")); then
   echo "2.Main IP : ${MAIN_IP}"
fi
if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "3.Main Share IP : ${MAIN_SHARE_IP}"
fi
if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "4.Hostname : ${SERVER_HOSTNAME}"
fi
if (("${host_use_user}" == "1")); then
   echo "5.第一組 DNS 主機名稱 : ${NS0}"
fi
if (("${host_use_user}" == "1")); then
   echo "6.第二組 DNS 主機名稱 : ${NS1}"
fi
if (("${host_use_user}" == "2")); then
   echo "7.接收 System 通知郵件(管理員信箱) : ${CUSTOMER_EMAIL}"
fi
if (("${host_use_user}" == "2")); then
   echo "8.接收 cPanel 通知郵件(管理員信箱) : ${WHM_EMAIL}"
fi
if [ "${virtualhost_type}" != "python" ]; then
   echo "9.PHP版本 : ${eapver_version}"
fi
#if (("${host_use_user}" == "1")); then
#   echo "10.安裝 Openresty (WP主機無須安裝) : ${installnginx}"
#fi
if (("${host_use_user}" == "1")); then
   echo "11.安裝 Default.htm 帳號預設頁面 : ${installindex}"
fi
#if (("${host_use_user}" == "1")); then
#   echo "12.安裝 Let's Encrypt cPanel Plugin : ${installletsencrypt}"
#fi
if (("${host_use_user}" == "1")); then
   echo "13.安裝本公司台灣或美國方案 : ${installpackages}"
fi
if (("${host_use_user}" == "1")); then
   echo "14.安裝 CXS : ${installcxs}"
fi
if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   #echo "15.安裝 CageFS、PHP Selector、MySQL Governor (Cloudlinux) : ${installcagefs}"
   echo "15.安裝 CageFS、PHP Selector (Cloudlinux) : ${installcagefs}"
fi
if [ "${host_use_user}" == "1" ] && [ "${virtualhost_type}" != "python" ]; then
   echo "16.安裝 Installatron : ${installinstallatron}"
fi
if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "17.安裝 KernelCare : ${kernelupdaten}"
fi
if (("${host_use_user}" == "1")); then
   echo "18.安裝 Zabbix : ${zabbix}"
fi
if (("${host_use_user}" == "2")); then
   echo "19.每小時寄信限制 : ${mail_qty}"
fi
if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "20.資料庫類型 : ${mysql_version}"
fi
if [ "${virtualhost_type}" != "python" ]; then
   echo "21.安裝 lsapi : ${lsapi}"
fi
#if (("${host_use_user}" == "1")); then
#   echo "22.虛擬主機類型 : ${virtualhost_type}"
#fi
#if [ "${host_use_user}" == "1" ] && [ "${virtualhost_type}" = "python" ] && [ "${installcagefs}" == "yes" ]; then
#   echo "23.開放使用者 SSH 登入 : ${allow_user_ssh}"
#fi
   echo "                                                                        "
   echo "  輸入項目編號，即可修改選項設定                                        "
   echo "                                                                        "
   echo "  (y) 開始安裝 cPanel (n) 取消安裝 cPanel                               "
   echo "                                                                        "
   echo "========================================================================"
   read -e -p "您的選擇是 ?  " -i "" ans
   case $ans in
        1)
           clear
           echo ""
           echo "### 請改成正確的外網網卡名稱 ###"
           echo ""
           read -e -p "1.外網網卡名稱 : " -i "$CPANEL_INTERFACE" CPANEL_INTERFACE
           continue;;
        2)
           clear
           echo ""
           read -e -p "2.Main IP : " -i "$MAIN_IP" MAIN_IP
           continue;;
        3)
           clear
           echo ""
           echo "### 管理服務主機，請設定客戶指定的 Main Share IP ###"
           echo ""
           read -e -p "3.Main Share IP : " -i "" MAIN_SHARE_IP
           continue;;
        4)
           clear
           echo ""
           echo "虛擬主機 cpX.g-dns.com(台灣)、半專屬主機 hdX.g-dns.com(台灣)"
           echo "經銷主機 reX.cx901.com(台灣) 或 urX.cx901.com(美國)"
           echo "WP主機 wpX.g-dns.com(台灣) 或 uwpX.g-dns.com(美國)"
           echo "貼圖主機 phX.g-dns.com(台灣)、"
           echo "新加坡主機 sgX.g-dns.com、香港主機 hkX.g-dns.com"
           echo "日本主機 jpX.g-dns.com"
           echo ""
           echo "### 管理服務主機，請依照管理服務安裝需求表設定 ###"
           echo ""
           read -e -p "4.Hostname : " -i "$SERVER_HOSTNAME" SERVER_HOSTNAME
           continue;;
        5)
           clear
           echo ""
           echo "虛擬主機 nsX.g-dns.com、經銷主機 nsX.cx901.com"
           echo ""
           read -e -p "5.第一組 DNS 主機名稱 : " -i "" NS0
           continue;;
        6)
           clear
           echo ""
           echo "虛擬主機 nsX.g-dns.com、經銷主機 nsX.cx901.com"
           echo ""
           read -e -p "6.第二組 DNS 主機名稱 : " -i "" NS1
           continue;;
        7)
           clear
           echo ""
           echo "### 請設定 Linux 需求表 管理員信箱 ###"
           echo ""
           read -e -p "7.接收 System 通知郵件(管理員信箱) : " -i "$CUSTOMER_EMAIL" CUSTOMER_EMAIL
           continue;;
        8)
           clear
           echo ""
           echo "### 請設定 Linux 需求表 管理員信箱 ###"
           echo ""
           read -e -p "8.接收 cPanel 通知郵件(管理員信箱) : " -i "$WHM_EMAIL" WHM_EMAIL
           continue;;
        9)
           clear
           echo ""
           echo "### 請依照管理服務安裝需求表設定 ###"
           echo ""
           echo "Apache 2.4 / PHP 7.2 / cgi = 72"
           echo "Apache 2.4 / PHP 7.3 / cgi = 73"
           echo "Apache 2.4 / PHP 7.4 / cgi = 74"
           echo "Apache 2.4 / PHP 8.0 / cgi = 80"
           echo "Apache 2.4 / PHP 8.1 / cgi = 81"
           echo "Apache 2.4 / PHP 8.2 / cgi = 82"
	   echo "Apache 2.4 / PHP 8.3 / cgi = 83"
           echo ""
           echo "例如 : PHP 7.2，就是輸入 72"
           echo ""
           read -e -p "9.PHP版本 : " -i "$eapver_version" eapver_version
           continue;;
#        10)
#           clear
#           echo ""
#           echo "安裝 = yes、不安裝 = no"
#           echo ""
#           echo "### 目前管理服務主機不提供安裝，原因為 https 抓取 SSL憑證 問題尚未解決 ###"
#           echo ""
#           read -e -p "10.安裝 Openresty : " -i "$installnginx" installnginx
#           continue;;
        11)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           read -e -p "11.安裝 Default.htm 帳號預設頁面 : " -i "$installindex" installindex
           continue;;
#        12)
#           clear
#           echo ""
#           echo "安裝 = yes、不安裝 = no"
#           echo ""
#           read -e -p "12.安裝 Let's Encrypt cPanel Plugin : " -i "$installletsencrypt" installletsencrypt
#           continue;;
        13)
           clear
           echo ""
           echo "台灣 = tw、美國 = usa、不安裝 = no"
           echo "亞洲虛擬主機，請直接複製目前現有主機的方案設定檔"
           echo ""
           read -e -p "13.安裝本公司台灣或美國方案 : " -i "" installpackages
           continue;;
        14)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           read -e -p "14.安裝 CXS : " -i "$installcxs" installcxs
           continue;;
        15)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           #read -e -p "15.安裝 CageFS、PHP Selector、MySQL Governor (Cloudlinux) : " -i "$installcagefs" installcagefs
           read -e -p "15.安裝 CageFS、PHP Selector (Cloudlinux) : " -i "$installcagefs" installcagefs
           continue;;
        16)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           read -e -p "16.安裝 Installatron : " -i "$installinstallatron" installinstallatron
           continue;;
        17)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           echo "### Standard 管理服務客戶才需要安裝 ###"
           echo ""
           read -e -p "17.安裝 KernelCare : " -i "$kernelupdaten" kernelupdaten
           continue;;
        18)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           echo "### 單次部屬不安裝，Standard 管理服務要安裝 ###"
           echo ""
           read -e -p "18.安裝 Zabbix : " -i "$zabbix" zabbix
           continue;;
        19)
           clear
           echo ""
           echo "### 管理服務主機，請依照管理服務安裝需求表設定 ###"
           echo ""
           read -e -p "19.每小時寄信限制 : " -i "$mail_qty" mail_qty
           continue;;
        20)
           clear
           echo ""
           echo "### MySQL 請輸入 8.0 ###"
           echo "### MariaDB 請輸入 10.3 or 10.5 or 10.6 or 10.11 ###"
           echo ""
           read -e -p "20.資料庫類型 : " -i "$mysql_version" mysql_version
           continue;;
        21)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           echo "自用虛擬主機 : 目前統一使用 lsapi php執行模式"
           echo "客戶主機 : 目前cPanel與Cloudlinux都有提供lsapi套件，目前統一使用 lsapi php執行模式"
           echo ""
           read -e -p "21.安裝 lsapi : " -i "$lsapi" lsapi
           continue;;
#	22)
#           clear
#           echo ""
#           echo "請選擇安裝的虛擬主機類型"
#		   echo "PHP = php、Python = python"
#           echo ""
#           read -e -p "22.虛擬主機類型 : " -i "$virtualhost_type" virtualhost_type
#           continue;;
#	23)
#           clear
#           echo ""
#                   echo "目前僅提供於Python虛擬主機使用"
#		   echo "開放 = yes、不開放 = no"
#           echo ""
#           read -e -p "23.是否開放使用者 SSH 登入 : " -i "$allow_user_ssh" allow_user_ssh
#           continue;;
        y)
           echo ""
           echo "開始安裝 cPanel"

### 判斷輸入值是否正確，不正確不給安裝 ###

if [ "${MAIN_SHARE_IP}" = "請手動輸入正確資訊" ]; then
echo ""
echo "3.Main Share IP : 請手動輸入正確資訊"
sleep 3
continue
fi

if [ "${NS0}" = "請手動輸入正確資訊" ]; then
echo ""
echo "5.第一組 DNS 主機名稱 : 請手動輸入正確資訊"
sleep 3
continue
fi

if [ "${NS1}" = "請手動輸入正確資訊" ]; then
echo ""
echo "6.第二組 DNS 主機名稱 : 請手動輸入正確資訊"
sleep 3
continue
fi

if [ "${CUSTOMER_EMAIL}" = "請手動輸入正確資訊" ]; then
echo ""
echo "7.接收 cPanel 日誌郵件的電子郵件 : 請手動輸入正確資訊"
sleep 3
continue
fi

if [ "${WHM_EMAIL}" = "請手動輸入正確資訊" ]; then
echo ""
echo "8.提供聯絡信息的電子郵件 : 請手動輸入正確資訊"
sleep 3
continue
fi

if [ "${eapver_version}" != '72' ] && [ "${eapver_version}" != '73' ] && [ "${eapver_version}" != '74' ] && [ "${eapver_version}" != '80' ] && [ "${eapver_version}" != '81' ] && [ "${eapver_version}" != '82' ] && [ "${eapver_version}" != '83' ]; then
echo ""
echo "9.目前輸入 PHP 版本為 ${eapver_version} 無法使用，請重新輸入正確值"
sleep 3
continue
fi

if [ "${installpackages}" != "tw" ] && [ "${installpackages}" != "usa" ] && [ "${installpackages}" != "no" ]; then
echo ""
echo "13.安裝本公司台灣或美國方案，請手動輸入正確資訊"
sleep 3
continue
fi

if [ "${mysql_version}" != "8.0" ] && [ "${mysql_version}" != "10.3" ] && [ "${mysql_version}" != "10.5" ] && [ "${mysql_version}" != "10.6" ] && [ "${mysql_version}" != "10.11" ]; then
echo ""
echo "20.目前輸入 mysql 版本 ${mysql_version} 無法使用，請重新輸入正確值"
sleep 3
continue
fi

if [ "${lsapi}" != "yes" ] && [ "${lsapi}" != "no" ]; then
echo ""
echo "21.是否安裝 lsapi 選項 ${lsapi} 無法使用，請重新輸入正確值"
sleep 3
continue
fi

#if [ "${virtualhost_type}" != "php" ] && [ "${virtualhost_type}" != "python" ]; then
#echo ""
#echo "22.目前輸入 虛擬主機類型 ${virtualhost_type} 無法使用，請重新輸入正確值"
#sleep 3
#continue
#fi

#if [ "${allow_user_ssh}" != "yes" ] && [ "${allow_user_ssh}" != "no" ]; then
#echo ""
#echo "23.是否開放使用者 SSH 登入選項 ${allow_user_ssh} 無法使用，請重新輸入正確值"
#sleep 3
#continue
#fi

           break;;
        n)
           echo ""
           echo "取消安裝 cPanel !!!"
           echo ""
           exit;;
        *)
           echo ""
           echo "輸入選擇為不正確 : $ans，請參考以下說明，輸入正確值"
           echo ""
           echo "(y) 開始安裝 cPanel (n) 取消安裝 cPanel"
           echo ""
           echo "或者 輸入項目編號，即可修改選項設定"
           sleep 3
           continue;;
   esac
done

# 紀錄選項設定值
echo " " >> /root/install_date
echo "######################"
echo "0.主機使用者類型：${host_use_user}" >> /root/install_date
echo "1.外網網卡名稱 : ${CPANEL_INTERFACE}" >> /root/install_date
echo "2.Main IP : ${MAIN_IP}" >> /root/install_date
echo "3.Main Share IP : ${MAIN_SHARE_IP}" >> /root/install_date
echo "4.Hostname : ${SERVER_HOSTNAME}" >> /root/install_date
echo "5.第一組 DNS 主機名稱 : ${NS0}" >> /root/install_date
echo "6.第二組 DNS 主機名稱 : ${NS1}" >> /root/install_date
echo "7.接收 cPanel 日誌郵件的電子郵件 : ${CUSTOMER_EMAIL}" >> /root/install_date
echo "8.提供聯絡信息的電子郵件 : ${WHM_EMAIL}" >> /root/install_date
echo "9.PHP版本 : ${eapver_version}" >> /root/install_date
#echo "10.安裝 Openresty : ${installnginx}" >> /root/install_date
echo "11.安裝 Default.htm 帳號預設頁面 : ${installindex}" >> /root/install_date
#echo "12.安裝 Let's Encrypt cPanel Plugin : ${installletsencrypt}" >> /root/install_date
echo "13.安裝本公司台灣或美國方案 : ${installpackages}" >> /root/install_date
echo "14.安裝 CXS : ${installcxs}" >> /root/install_date
echo "15.安裝 CageFS、PHP Selector、MySQL Governor (Cloudlinux) : ${installcagefs}" >> /root/install_date
echo "16.安裝 Installatron : ${installinstallatron}" >> /root/install_date
echo "17.安裝 KernelCare : ${kernelupdaten}" >> /root/install_date
echo "18.安裝 Zabbix : ${zabbix}" >> /root/install_date
echo "19.每小時寄信限制 : ${mail_qty}" >> /root/install_date
echo "20.資料庫類型 : ${mysql_version}" >> /root/install_date
echo "21.安裝 lsapi : ${lsapi}" >> /root/install_date
#echo "22.虛擬主機類型 : ${virtualhost_type}" >> /root/install_date
#echo "23.開放使用者 SSH 登入 : ${allow_user_ssh}" >> /root/install_date
echo "######################"
echo " " >> /root/install_date



##### 開始安裝 cPanel #####



# 設定正確 hostname
/usr/bin/hostnamectl set-hostname $SERVER_HOSTNAME

# 再次向 CloudLinux Server 註冊 (如果系統是 CentOS，會跳過註冊)
if [ "$CT" ]; then
  /usr/sbin/clnreg_ks --force
fi

# 設定 cPanel 使用更新為 stable
echo "CPANEL=stable" > /etc/cpupdate.conf

# 複製 cpanel.config 範例設定檔
mkdir -p /root/cpanel_profile
cp /opt/cpinst/cpinst/config/cpanel.config /root/cpanel_profile/

# 客戶主機重啟Apache服務間隔改成預設值10秒
if [ "$host_use_user" == "2" ]; then
sed -i "s/^min_time_between_apache_graceful_restarts=.*/min_time_between_apache_graceful_restarts=10/g" /root/cpanel_profile/cpanel.config
fi

# 設定每小時寄信最大限制數量 (客戶主機 才會進行設定，自用主機都是使用預設值 300)
if [ "$host_use_user" == "2" ]; then
  sed -i "s/^maxemailsperhour=.*/maxemailsperhour=${mail_qty}/g" /root/cpanel_profile/cpanel.config
fi

# 設定資料庫類型
sed -i "s/^mysql-version=.*/mysql-version=${mysql_version}/g" /root/cpanel_profile/cpanel.config

# 設定帳戶開啟是否預設要使用 jailshell
if [ "$allow_user_ssh" == "yes" ]; then
   sed -i "s/^jaildefaultshell=.*/jaildefaultshell=1/g" /root/cpanel_profile/cpanel.config
fi

# 複製 EA4 Json 範例設定檔 
if [ "${virtualhost_type}" == "php" ]; then
   eapver="cpanel_install_ea4_profile_php_${eapver_version}.json"
   cp /opt/cpinst/cpinst/config/${eapver} /etc/cpanel_initial_install_ea4_profile.json
#elif [ "${virtualhost_type}" == "python" ]; then
#   cp /opt/cpinst/cpinst/config/cpanel_install_ea4_profile_python.json /etc/cpanel_initial_install_ea4_profile.json
fi

# 避免因為非使用cloudlinux系統，部分套件無法安裝，導致客製化json檔案套用失敗 Ex.2023/11/7
if [ -e /usr/local/src/no_install_cloudlinux ]; then
  sed -i '/ea-libcurl/d' /etc/cpanel_initial_install_ea4_profile.json
  sed -i '/ea-libcurl-devel/d' /etc/cpanel_initial_install_ea4_profile.json
  sed -i '/ea-apache24-mod_hostinglimits/d' /etc/cpanel_initial_install_ea4_profile.json
  sed -i '/ea-openssl/d' /etc/cpanel_initial_install_ea4_profile.json
  sed -i '/ea-openssl-devel/d' /etc/cpanel_initial_install_ea4_profile.json
fi

# 如果沒有安裝 Openresty，Apache port 置換 28080 to 80 和 28443 to 443
#if [ "$installnginx" == "no" ]; then
#  sed -i 's/apache_port=.*/apache_port=0.0.0.0:80/g' /root/cpanel_profile/cpanel.config
#  sed -i 's/apache_ssl_port=.*/apache_ssl_port=0.0.0.0:443/g' /root/cpanel_profile/cpanel.config
#fi

# 複製 wwwacct.conf 範例設定檔，置換正確的設定
cp /opt/cpinst/cpinst/config/wwwacct.conf /etc/wwwacct.conf 
sed -i "s/QQHSTNME/${SERVER_HOSTNAME}/g" /etc/wwwacct.conf
sed -i "s/QQINTERFACE/${CPANEL_INTERFACE}/g" /etc/wwwacct.conf
sed -i "s/QQWHMMAIL/${WHM_EMAIL}/g" /etc/wwwacct.conf
sed -i "s/QQMAINSHAREIP/${MAIN_SHARE_IP}/g" /etc/wwwacct.conf
sed -i "s/QQNAM1/${NS0}/g" /etc/wwwacct.conf
sed -i "s/QQNAM2/${NS1}/g" /etc/wwwacct.conf

# 開始安裝 cPanel，安裝紀錄存放 /var/log/cpanel-install.log
cd /root && curl -o latest -L https://securedownloads.cpanel.net/latest && sh latest

# 更新 cPanel license
/usr/local/cpanel/cpkeyclt

# 關閉不必要的服務
cat <<'EOF_DISABLE_SERVICES' > /root/cpanel-disable-services
#!/bin/bash
for service in \
   avahi-daemon \
   avahi-dnsconfd \
   bluetooth \
   capi \
   conman \
   cpus \
   cups \
   irda \
   lisa \
   netplugd \
   ifplugd \
   NetworkManager \
   NetworkManagerDispatcher \
   netfs \
   netplugd \
   nfs \
   nfslock \
   nfsd \
   ncsd \
   oddjobd \
   openvpn \
   pcmcia \
   pcscd \
   portsentry \
   postfix \
   rawdevices \
   rdisc \
   rpcgssd \
   rpcidmapd \
   saslauthd \
   tcsd \
   vsftpd \
   winbind \
   wpa_supplicant \
   xfs \
   mdmpd \
   mdmonitor \
   canna \
   FreeWnn \
   cups-config-daemon \
   iiim \
   mDNSResponder \
   nifd \
   anacron \
   gpm \
   hidd \
   sbadm \
   qpidd \
   ypbind ;

   do
       echo "Disabling service: ${service}"
       service ${service} stop &>/dev/null
       systemctl stop ${service} &>/dev/null
       systemctl disable ${service} &>/dev/null
done
EOF_DISABLE_SERVICES
chmod 700 /root/cpanel-disable-services
/root/cpanel-disable-services
rm -f /root/cpanel-disable-services

# 安裝 Account DNS Check cPanel/WHM Plugin
#cd /usr/local/src
#wget http://download.ndchost.com/accountdnscheck/latest-accountdnscheck
#sh latest-accountdnscheck
#rm -f /usr/local/src/latest-accountdnscheck
#rm -rf /usr/local/src/installd-accountdnscheck

# 增加 cPanel theme style
GITBIN="/usr/local/cpanel/3rdparty/bin/git"
mkdir -p /var/cpanel/customizations/styled/
cd /var/cpanel/customizations/styled/
$GITBIN clone https://github.com/longster/cP-Starter-Dark.git
$GITBIN clone https://github.com/longster/cP-Starter-Light.git

# register appconfig for csf.php & restoreacct.conf
# check command : whmapi1 get_appconfig_application_list
if [ "$host_use_user" == "1" ]; then
/usr/local/cpanel/bin/register_appconfig /opt/cpinst/cpinst/config/unblockip.conf
mkdir -p /usr/local/cpanel/whostmgr/docroot/cgi/unblockip/
/bin/cp -f /opt/cpinst/moni_script/csf.php /usr/local/cpanel/whostmgr/docroot/cgi/unblockip/
#/usr/local/cpanel/bin/register_appconfig /opt/cpinst/cpinst/config/restoreacct.conf
fi

# 安裝 CSF 套件
sh /opt/cpinst/cpinst/install_script/csf.sh ${host_use_user}

# 開啟 Shell Fork Bomb Protection (與CageFS會有衝突)
#perl -I/usr/local/cpanel -MCpanel::LoginProfile -le 'print [Cpanel::LoginProfile::install_profile('limits')]->[1];'

# 安裝 Openresty (包含 RemoteIP 套件)
#if [ "$installnginx" == "yes" ]; then
#  /bin/bash /opt/cpinst/cpinst/install_script/openresty.sh
#  /bin/bash /opt/cpinst/cpinst/install_script/remoteip.sh  
#fi

# 安裝 RemoteIP 套件
if [ "$host_use_user" == "1" ]; then
/bin/bash /opt/cpinst/cpinst/install_script/remoteip.sh
fi
if [ "$host_use_user" == "2" ]; then
/bin/bash /opt/cpinst/cpinst/install_script/remoteip.sh
sed -i '/add_remoteip/d' /etc/crontab
cp /opt/cpinst/cpinst/install_script/add_remoteip.sh /root/add_remoteip.sh
/bin/bash /root/add_remoteip.sh
echo "30 0 * * * root /bin/bash /root/add_remoteip.sh > /dev/null 2>&1" >> /etc/crontab
fi

# 安裝 Default.htm 帳號預設頁面
# 如果預設要讓網站開啟 gzip 壓縮，可建立 .htaccess 於預設帳號目錄內，加入以下網頁內說明的語法 (此設定尚未套用，往後有需要再說)
# https://gtmetrix.com/enable-gzip-compression.html
if [ "$installindex" == "yes" ]; then
  mkdir -p /root/cpanel3-skel/public_html
  cp /opt/cpinst/cpinst/config/Default.htm /root/cpanel3-skel/public_html
fi

# 安裝 Let's Encrypt cPanel Plugin
#if [ "$installletsencrypt" == "yes" ]; then
#  wget -O /etc/yum.repos.d/letsencrypt-cpanel.repo http://g-host.net/gay/scripts/letsencrypt-cpanel/letsencrypt-cpanel.repo
#  wget -O /etc/letsencrypt-cpanel.licence http://g-host.net/gay/scripts/letsencrypt-cpanel/letsencrypt-cpanel.licence
#  chmod 0400 /etc/letsencrypt-cpanel.licence
#  yum install letsencrypt-cpanel -y
#  sed -i 's/\"disablerenewalmail\": false/\"disablerenewalmail\": true/g' /etc/letsencrypt-cpanel.conf
#  sed -i 's/\"disable_success_mail\": false/\"disable_success_mail\": true/g' /etc/letsencrypt-cpanel.conf
#  service letsencrypt-cpanel restart
#  mainip=`cat /var/cpanel/mainip`
#  lets=`whmapi1 api_token_list | grep 'name: letsencrypt' | awk '{print $2}'` ; whmapi1 api_token_update token_name="$lets" whitelist_ip="$mainip"
#fi

# 啟用 Two-Factor Authentication
whmapi1 twofactorauth_enable_policy

# 安裝本公司台灣或美國方案
if [ "$installpackages" == "tw" ]; then
  /bin/cp -f /opt/cpinst/cpinst/config/packages/tw/* /var/cpanel/packages/
fi
if [ "$installpackages" == "usa" ]; then
  /bin/cp -f /opt/cpinst/cpinst/config/packages/usa/* /var/cpanel/packages/
fi

# 安裝 CXS
if [ "$installcxs" == "yes" ]; then
  /bin/bash /opt/cpinst/cpinst/install_script/cxs.sh
fi

# 安裝 CageFS、MySQL Governor
if [ "$installcagefs" == "yes" ]; then
  /bin/bash /opt/cpinst/cpinst/install_script/cagefs.sh

# mysql governor 目前改手動安裝，因為失敗率太高
#  yum install governor-mysql -y

#if [ "$mysql_version" == "8.0" ]; then
#/usr/share/lve/dbgovernor/db-select-mysql --mysql-version=mysql80
#/usr/share/lve/dbgovernor/mysqlgovernor.py --install-beta
#fi
#if [ "$mysql_version" == "10.6" ]; then
#/usr/share/lve/dbgovernor/db-select-mysql --mysql-version=mariadb106
#/usr/share/lve/dbgovernor/mysqlgovernor.py --install
#fi

#  /bin/bash /opt/cpinst/cpinst/install_script/mysqlgovernor.sh
  touch /etc/container/cl-quotas.dat
fi

# 安裝 PHP Selector or Python Selector
if [ "${virtualhost_type}" == "php" ] && [ "$installcagefs" == "yes" ]; then 
   /bin/bash /opt/cpinst/cpinst/install_script/phpselector.sh
#elif [ "${virtualhost_type}" == "python" ]; then
#   /bin/bash /opt/cpinst/cpinst/install_script/pythonselector.sh
fi

# 安裝 Installatron
if [ "$installinstallatron" == "yes" ]; then
  rpm -Uvh http://dal01.installatron.com/installatron-plugin-cpanel-latest.noarch.rpm
fi

# 安裝 KernelCare
if [ "$kernelupdaten" == "yes" ]; then
  curl -s https://repo.cloudlinux.com/kernelcare/kernelcare_install.sh | bash
if [ "$CT" ]; then
/usr/bin/cat <<EOF >> /etc/sysctl.conf
 ## protect against symlink attack
fs.enforce_symlinksifowner=1
EOF
kcarectl --set-patch-type extra --update
/usr/sbin/sysctl -p
fi
kcarectl --update
fi

# 安裝 Cloudflare
# 目前 ea-apache24-mod_cloudflare 套件由cloudlinux提供，無法提供給沒有安裝cloudlinux主機使用
#if [ "$installcagefs" == "yes" ]; then
#cd /opt/
#wget  https://raw.githubusercontent.com/cloudflare/mod_cloudflare/master/EasyApache/installer.sh
#sh installer.sh
#wget https://raw.githubusercontent.com/cloudflare/CloudFlare-CPanel/master/cloudflare.install.sh
#sh cloudflare.install.sh -k 5b0e46d5765f50ca014eb00f13da4ef0 -n 'Yuan-Jhen Info., Co., Ltd.'
# 有時會安裝失敗，所以跑兩次
#sh cloudflare.install.sh -k 5b0e46d5765f50ca014eb00f13da4ef0 -n "Yuan-Jhen Info., Co., Ltd."
#yum install ea-apache24-mod_cloudflare -y
#/bin/cp -f /opt/cpinst/cpinst/config/cloudflare.conf /etc/apache2/conf.d/
#/scripts/restartsrv_httpd
#rm -f installer.sh
#rm -f cloudflare.install.sh
#fi

# 關閉 Compiler Access
if (("${virtualhost_type}" == "php")); then 
   /scripts/compilers off
#elif (("${virtualhost_type}" == "python")); then
#   /scripts/compilers on
fi

# 允許使用者使用SSH，安裝第二個sshd，僅應用於Python虛擬主機
if [ "$allow_user_ssh" == "yes" ]; then
   /bin/bash /opt/cpinst/cpinst/install_script/ssh_login.sh
fi

# 設定 Edit System Mail Preferences
echo "cpanel: ${CUSTOMER_EMAIL}" > /etc/localaliases
echo 'nobody: /dev/null' >> /etc/localaliases
echo "root: ${CUSTOMER_EMAIL}" >> /etc/localaliases

# 更新 /etc/my.cnf 設定檔
mv /etc/my.cnf /etc/my.cnf_bak
if [ "$mysql_version" == "8.0" ]; then
  cp /opt/cpinst/cpinst/config/my_8.txt /etc/my.cnf
fi
if [ "$mysql_version" == "10.3" ] && [ "${mysql_version}" != "10.5" ] && [ "${mysql_version}" != "10.6" ] && [ "${mysql_version}" != "10.11" ]; then
  cp /opt/cpinst/cpinst/config/my.txt /etc/my.cnf
  sed -i '/optimizer_switch/d' /etc/my.cnf
fi
/scripts/restartsrv_mysql
ln -s /var/lib/mysql/mysql.err /var/lib/mysql/`hostname`.err

# 抓資源限制之設定檔
if [ "$host_use_user" == "1" ]; then
if [ -e /etc/container/ve.cfg ]; then
  /bin/mv -f /etc/container/ve.cfg /etc/container/ve.cfg_bak
  /bin/cp -f /opt/cpinst/cpinst/config/phpselector/ve_cfg /etc/container/ve.cfg
  lvectl set default --vmem=0
  lvectl apply all
fi
fi

# 抓inode限制之設定檔
if [ "$host_use_user" == "1" ]; then
if [ -e /etc/container/cl-quotas.dat ]; then
  /bin/mv -f /etc/container/cl-quotas.dat /etc/container/cl-quotas.dat_bak
  /bin/cp -f /opt/cpinst/cpinst/config/phpselector/cl-quotas.dat /etc/container/cl-quotas.dat
  /usr/bin/cl-quota -YC
fi
fi

# 複製 Contact Manager 範例設定檔
mv /var/cpanel/icontact_event_importance.json /var/cpanel/icontact_event_importance.json_bak
/bin/cp -f /opt/cpinst/cpinst/config/icontact_event_importance.json /var/cpanel/icontact_event_importance.json
/bin/cp -f /opt/cpinst/cpinst/config/clevels.conf /var/cpanel/clevels.conf

# Apache Configuration > Global Configuration
#sed -i "s/startservers: 5/startservers: 10/g" /var/cpanel/conf/apache/local
#sed -i "s/maxclients: 100/maxclients: 256/g" /var/cpanel/conf/apache/local
#sed -i "s/maxclients: 150/maxclients: 256/g" /var/cpanel/conf/apache/local
#sed -i 's/%h %l %u %t \\\\\\\"%r\\\\\\\" %>s %b \\\\\\\"%{Referer}i\\\\\\\" \\\\\\\"%{User-Agent}i\\\\\\\"/%a %l %u %t \\\\\\\"%r\\\\\\\" %>s %b \\\\\\\"%{Referer}i\\\\\\\" \\\\\\\"%{User-Agent}i\\\\\\\" %I %O/g' /var/cpanel/conf/apache/local

# Apache Configuration > Include Editor
if (("${virtualhost_type}" == "php")); then

:<<!
/usr/bin/cat <<EOF >> /usr/local/apache/conf/includes/post_virtualhost_global.conf
<IfModule mod_fcgid.c>
# scan interval for busy timeout process
FcgidBusyScanInterval 90

# a FastCGI application will be killed after handling a request for FcgidBusyTimeout
FcgidBusyTimeout 600

# scan interval for exited process
FcgidErrorScanInterval 3

# Mirror the PHP cgi.fix_pathinfo setting
FcgidFixPathinfo 1

# scan interval for idle timeout process
FcgidIdleScanInterval 70

# An idle FastCGI application will be killed after FcgidIdleTimeout
FcgidIdleTimeout 360

# Communication timeout to FastCGI server
FcgidIOTimeout 1000

# maximum number of FastCGI application processes
FcgidMaxProcesses 1000

# Max process count of one class of FastCGI application
FcgidMaxProcessesPerClass 100

# maximum size of a request which will be held in memory
FcgidMaxRequestInMem 268435456

# maximum HTTP request length (128M)
FcgidMaxRequestLen 134217728

# Max requests handled by each FastCGI application
FcgidMaxRequestsPerProcess 0

# Min process count of one class of FastCGI application
FcgidMinProcessesPerClass 3

# CGI output buffer size
FcgidOutputBufferSize 1048576

# Header name which will be passed to FastCGI as environment variable
FcgidPassHeader HTTP_AUTHORIZATION

# maximum FastCGI application process lifetime
FcgidProcessLifeTime 300

# Each spawn adds this value to the process activity score
FcgidSpawnScore 1

# Maximum value of the process activity score which allows a spawn to occur
FcgidSpawnScoreUpLimit 10

# Each terminated process adds this value to the process activity score.
FcgidTerminationScore 2

# Amount subtracted from process activity score each second
FcgidTimeScore 2

# scan interval for zombie process
FcgidZombieScanInterval 3
</IfModule>
EOF
!

yum install ea-apache24-mod_reqtimeout -y
/usr/bin/cat <<EOF >> /usr/local/apache/conf/includes/pre_main_global.conf
<IfModule mod_reqtimeout.c>
RequestReadTimeout header=10-20,MinRate=500 body=20-600,MinRate=500
</IfModule>
EOF
/usr/bin/cat <<EOF >> /etc/apache2/conf.d/includes/pre_virtualhost_global.conf
SSLUseStapling off
EOF
/scripts/rebuildhttpdconf
/scripts/restartsrv_httpd
fi

# Exim Configuration Manager
# https://docs.cpanel.net/whm/scripts/the-cpconftool-script/82/
mkdir /var/cpanel/config.backups/
/bin/cp /opt/cpinst/cpinst/config/whm-config-backup-cpanel__smtp__exim-116.001000-1710249805.tar.gz /var/cpanel/config.backups/
/usr/local/cpanel/bin/cpconftool --restore=/var/cpanel/config.backups/whm-config-backup-cpanel__smtp__exim-116.001000-1710249805.tar.gz --modules=cpanel::smtp::exim
echo '*: '`cat /var/cpanel/mainip` > /etc/mailips
echo '*: '`hostname` > /etc/mailhelo

# pure-ftpd
/scripts/setupftpserver --force pure-ftpd
rm -f /var/cpanel/conf/pureftpd/main.cache
mv /var/cpanel/conf/pureftpd/main /var/cpanel/conf/pureftpd/main.backup
/bin/cp -f /opt/cpinst/cpinst/config/pureftpd_main /var/cpanel/conf/pureftpd/main
touch /var/cpanel/conf/pureftpd/local
echo 'ForcePassiveIP: ~' >> /var/cpanel/conf/pureftpd/local
echo 'PassivePortRange: 30000 50000' >> /var/cpanel/conf/pureftpd/local
echo 'TLSCipherSuite: ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:DHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:TLS_AES_256_GCM_SHA384:TLS_CHACHA20_POLY1305_SHA256:TLS_AES_128_GCM_SHA256' >> /var/cpanel/conf/pureftpd/local
sed -i "s/PassivePortRange 49152 65534/PassivePortRange 30000 50000/g" /etc/pure-ftpd.conf
/scripts/setupftpserver --force pure-ftpd
/scripts/restartsrv_ftpserver

# 安裝 SNMP
cd /usr/local/src
wget -O snmp.sh https://cpanel.net.tw/tool/snmp/snmp.txt
sh /usr/local/src/snmp.sh

# 安裝 Memcached
/usr/bin/cat <<EOF >> /etc/yum.repos.d/EA4-experimental.repo
[EA4-experimental]
name=EA4 Experimental ( EasyApache 4 )
mirrorlist=http://httpupdate.cpanel.net/ea4-experimental-c\$releasever-\$basearch-mirrorlist
gpgcheck=1
gpgkey=https://securedownloads.cpanel.net/cPanelPublicRPMKey.asc
enabled=1
cost=50
EOF
yum clean all
yum install ea-memcached -y
systemctl daemon-reload
systemctl enable memcached.service
systemctl start memcached.service
systemctl enable ea-memcached.service
systemctl start ea-memcached.service
netstat -ntlp | grep memcached
if (("${virtualhost_type}" == "php")); then
   yum install ea-php${eapver_version}-php-memcache -y
   yum install ea-php${eapver_version}-php-memcached -y
fi

# 關閉 Core Dump
cat /etc/security/limits.conf  |grep -v  . -n -m 1 |cut -d : -f1 |xargs -i  sed -i  '{}c \\*\thard\tcore\t0' /etc/security/limits.conf
echo 'kernel.core_pattern = /dev/null/' >> /etc/sysctl.conf
echo 'fs.suid_dumpable = 0' >> /etc/sysctl.conf
sysctl -p
echo 'ulimit -S -c 0 > /dev/null 2>&1' >> /etc/profile

# 安裝 imagick
#if [ "$virtualhost_type" == "php" ]; then
#   # 安裝相關套件
#   yum -y install ImageMagick ImageMagick-devel
#   # 啟用 imagick
#   autodetect | /opt/cpanel/ea-php${eapver_version}/root/usr/bin/pecl install imagick
#   # 調整 easyapache4 native php.ini
#   sed -i 's/^expose_php.*.=.*/expose_php = off/g' /opt/cpanel/ea-php${eapver_version}/root/etc/php.ini
#   sed -i 's/^log_errors.*.=.*/log_errors = off/g' /opt/cpanel/ea-php${eapver_version}/root/etc/php.ini
#   sed -i 's/^enable_dl.*.=.*/enable_dl = Off/g' /opt/cpanel/ea-php${eapver_version}/root/etc/php.ini
#   sed -i 's/^disable_functions.*.=.*/disable_functions = symlink,show_source, system, exec, shell_exec, passthru, popen, proc_open, proc_close, proc_get_status, proc_nice, proc_terminate, escapeshellarg, escapeshellcmd, ini_alter, define_syslog_variables,posix_uname,posix_getpwuid,posix_getpwnam, apache_child_terminate,posix_kill,posix_mkfifo,posix_setpgid,posix_setsid,posix_setuid,proc_terminate, posix_uname, diskfreespace, leak,base64_encodem,eval, apache_child_terminate, apache_setenv/g' /opt/cpanel/ea-php${eapver_version}/root/etc/php.ini
#   sed -i 's/date\.timezone.*.=.*/date\.timezone="Asia\/Taipei"/g' /opt/cpanel/ea-php${eapver_version}/root/etc/php.ini
#   sed -i 's/memory_limit.*.=.*/memory_limit = 128M/g' /opt/cpanel/ea-php${eapver_version}/root/etc/php.ini
#   /usr/local/cpanel/bin/rebuild_phpconf --default=ea-php${eapver_version}
#fi

# 關閉 PHP-FPM
whmapi1 php_set_default_accounts_to_fpm default_accounts_to_fpm=0

# 安裝 Zabbix
if [ "$zabbix" == "yes" ]; then

cd /root
git clone git@git.twnoc.net:nicole/zabbix.git
sh /root/zabbix/zabbix_install.sh

if [ "$host_use_user" == "1" ]; then

if [ ! -e /home/phpchk ]; then

# 避免建立的帳號無法正常切換php版本，因此再次執行重裝套件以及更新設定 2019-9-4 Jacky
yum reinstall cagefs lvemanager lve-stats lve-utils -y
/bin/mv -f /etc/cl.selector/php.conf /etc/cl.selector/php.conf.backup
cp -f /opt/cpinst/cpinst/config/php_conf /etc/cl.selector/php.conf
sed -i 's/^expose_php.*.=.*/expose_php = Off/g' /opt/alt/php*/etc/php.ini
sed -i 's/^log_errors.*.=.*/log_errors = Off/g' /opt/alt/php*/etc/php.ini
sed -i 's/^enable_dl.*.=.*/enable_dl = Off/g' /opt/alt/php*/etc/php.ini
sed -i 's/^disable_functions.*.=.*/disable_functions = symlink,show_source, system, exec, shell_exec, passthru, popen, proc_open, proc_close, proc_get_status, proc_nice, proc_terminate, escapeshellarg, escapeshellcmd, ini_alter, define_syslog_variables,posix_uname,posix_getpwuid,posix_getpwnam, apache_child_terminate,posix_kill,posix_mkfifo,posix_setpgid,posix_setsid,posix_setuid,proc_terminate, posix_uname, diskfreespace, leak,base64_encodem,eval, apache_child_terminate, apache_setenv/g' /opt/alt/php*/etc/php.ini
sed -i 's/date\.timezone.*.=.*/date\.timezone="Asia\/Taipei"/g' /opt/alt/php*/etc/php.ini
sed -i 's/^;memory_limit.*.=.*/memory_limit = 128M/g' /opt/alt/php*/etc/php.ini
sed -i 's/^upload_max_filesize.*/upload_max_filesize = 32M/g' /opt/alt/php*/etc/php.ini
cagefsctl --remount-all
cagefsctl --force-update

/usr/bin/hostnamectl set-hostname $SERVER_HOSTNAME
/usr/local/cpanel/bin/set_hostname $SERVER_HOSTNAME
phpchk_domain=`hostname -s`.phpchk.com

whmapi1 createacct username=phpchk domain=${phpchk_domain}

fi

# PHP 虛擬主機新增 PHP Selector 與 remoteip 監控
if [ "$virtualhost_type" == "php" ]; then
#新增 php_select_test.php 程式
touch /home/phpchk/public_html/php_select_test.php
chown phpchk.phpchk /home/phpchk/public_html/php_select_test.php
cat << EOF > /home/phpchk/public_html/php_select_test.php
<?
echo phpversion();
?>
EOF

#新增 ip.php 程式
touch /home/phpchk/public_html/ip.php
chown phpchk.phpchk /home/phpchk/public_html/ip.php
cat << EOF > /home/phpchk/public_html/ip.php
<?php
echo \$_SERVER['REMOTE_ADDR'] . " " ;
echo \$_SERVER['SERVER_ADDR'] ;
?>
EOF

#新增 remoteip.php 程式
touch /home/phpchk/public_html/remoteip.php
chown phpchk.phpchk /home/phpchk/public_html/remoteip.php
mainip=`cat /var/cpanel/mainip`
cat << EOF > /home/phpchk/public_html/remoteip.php
<?php
\$ch = curl_init();
curl_setopt(\$ch,CURLOPT_URL,"http://${phpchk_domain}/ip.php");
curl_setopt(\$ch, CURLOPT_INTERFACE, "$mainip"); //ip 請填入此主機 main ip
curl_setopt(\$ch,CURLOPT_HEADER,0);//是否輸出header
\$out = curl_exec(\$ch);
curl_close(\$ch);
?>
EOF

fi

# Python 虛擬主機新增remoteip 監控
#if [ "$virtualhost_type" == "python" ]; then
#
## 新增 flask app 來接收 user real ip
#/usr/bin/selectorctl --interpreter=python --version=3.5 --user=phpchk --create-webapp python/phpchk /
#source /home/phpchk/virtualenv/python_phpchk/3.5/bin/activate
#pip3 install flask
#deactivate
#cd /home/phpchk/public_html
#\cp -f /opt/cpinst/cpinst/config/python_remoteip_monitor/passenger_wsgi.py /home/phpchk/python/phpchk/
#chown phpchk.phpchk /home/phpchk/python/phpchk/passenger_wsgi.py
#selectorctl --interpreter python --user phpchk --domain phpchk.com --restart-webapp python/phpchk
#
#fi

fi

fi

# Paper_lantern > 調整顯示功能
if [ "$host_use_user" == "1" ]; then
cd /var/cpanel/features/
echo "" > default
echo "" > photo
echo "" > single
echo "" > addon
echo "" > single-hd
echo "" > disabled
cat /opt/cpinst/cpinst/config/feature/default > 'default'
cat /opt/cpinst/cpinst/config/feature/photo > 'photo'
cat /opt/cpinst/cpinst/config/feature/single > 'single'
cat /opt/cpinst/cpinst/config/feature/addon > 'addon'
cat /opt/cpinst/cpinst/config/feature/single-hd > 'single-hd'
cat /opt/cpinst/cpinst/config/feature/disabled > 'disabled'

# 關閉MySQL遠端功能選項顯示
touch /usr/local/cpanel/base/frontend/jupiter/dynamicui/dynamicui_hide_icon.conf
echo "file=>remote_mysql,skipobj=>1" > \
/usr/local/cpanel/base/frontend/jupiter/dynamicui/dynamicui_hide_icon.conf
fi
#if [ "${virtualhost_type}" == "python" ]; then
#   sed -i "s/lvephpsel=1/lvephpsel=0/g" /var/cpanel/features/*
#   sed -i "s/lvepythonsel=0/lvepythonsel=1/g" /var/cpanel/features/*
   #replace 'lvephpsel=1' 'lvephpsel=0' -- /var/cpanel/features/*
   #replace 'lvepythonsel=0' 'lvepythonsel=1' -- /var/cpanel/features/*
#fi
if [ "$host_use_user" == "2" ]; then
whmapi1 update_featurelist featurelist='defauilt' zoneedit=1
fi

# cPanel面板更新翻譯
/opt/cpinst/cpinst/install_script/themes.sh

# 限制用戶切換面板
if [ "$host_use_user" == "1" ]; then
echo "jupiter" > /var/cpanel/themes.conf
fi

# 新增 postupcp，並設定成 upcp post hook 執行
# 經常造成upcp更新完畢卡住，經確認此程式已無太大用途，故取消使用 Ex.2023/6/15
#if [ "$host_use_user" == "1" ]; then
#mkdir -p /home/pre-scripts/conf
#mkdir -p /home/pre-scripts/scripts
#mkdir -p /home/pre-scripts/restore_tmp
#cd /scripts/
#cp /opt/cpinst/moni_script/postupcp8.sh postupcp
#chmod 0755 postupcp
#/scripts/postupcp
#/usr/local/cpanel/bin/manage_hooks add script /scripts/postupcp --manual --category System --event upcp --stage post
#fi

# 安裝 Safe-rm
if [ "$host_use_user" == "1" ]; then
/opt/cpinst/cpinst/install_script/safe-rm.sh
fi

# 幫 hostname 安裝 免費憑證
if [ "$host_use_user" == "1" ]; then
/usr/local/cpanel/bin/checkallsslcerts
fi

# 安裝 lsapi
if [ "$lsapi" == "yes" ]; then
/opt/cpinst/cpinst/install_script/lsapi.sh
fi

# 安裝 mod_fcgid，設定主機全部 PHP 版本 Handlers 改成 fcgid (僅針對客戶主機)
#if [ "${host_use_user}" == "2" ] && [ "${virtualhost_type}" == "php" ]; then
#/opt/cpinst/cpinst/install_script/fcgi.sh
#fi

# 關閉 cPHulk
whmapi1 configureservice service=cphulkd enabled=0 monitored=0

# 阻擋客戶網站被存取.git目錄與.git與.env檔案
cp /opt/cpinst/cpinst/config/deny_git.conf /etc/apache2/conf.d/
cp /opt/cpinst/cpinst/config/deny_env.conf /etc/apache2/conf.d/
/scripts/restartsrv_httpd

# 監控主機 sshd Port 22 是否有被 try
if [ "$host_use_user" == "1" ]; then
echo "0 * * * * root /bin/bash /opt/cpinst/jacky_script/check_sshd.sh" >> /etc/crontab
fi

# 監控主機 LOAD
if [ "$host_use_user" == "1" ]; then
echo "*/5 * * * * root /bin/bash /opt/cpinst/jacky_script/check_load.sh" >> /etc/crontab
fi

# 每日16:00執行SQL自動備份
if [ "$host_use_user" == "1" ]; then
mkdir /backup
touch /backup/sqldump_error.log
echo '0 16 * * * root script -c /opt/cpinst/jacky_script/daily_dump_all_sql.sh /backup/sqldump_error.log' >> /etc/crontab
fi

# 限制cPanel帳號無法存取內部網路
if [ "$host_use_user" == "1" ]; then
echo '0 */6 * * * root /bin/bash /opt/cpinst/jacky_script/cpanel_account_intranet_reject.sh --run > /dev/null 2>&1' >> /etc/crontab
fi

# fix not open webmail for roundcube problem
/usr/local/cpanel/bin/update-roundcube-db

# 自製 Suspend Page
if [ "$host_use_user" == "1" ]; then
mkdir -p /var/cpanel/webtemplates/root/english/
cp /opt/cpinst/cpinst/config/suspended.tmpl /var/cpanel/webtemplates/root/english/
cp /opt/cpinst/cpinst/config/moving.tmpl /var/cpanel/webtemplates/root/english/
fi

# Apache Worker偵測重啟程式 By.Webber
#dnf config-manager --set-enabled powertools
#yum install -y lynx
#/bin/cp /opt/cpinst/cpinst/scripts/apache_restart.sh /root/
#/bin/cp /opt/cpinst/cpinst/scripts/apache_start.sh /root/
#echo '/bin/bash /root/apache_start.sh start' >> /etc/rc.d/rc.local
#/bin/bash /root/apache_start.sh start

# 再次設定正確 hostname，避免DNS未指向，被cPanel改成xxxx.cprapid.com名稱
/usr/bin/hostnamectl set-hostname $SERVER_HOSTNAME
/usr/local/cpanel/bin/set_hostname $SERVER_HOSTNAME
echo '*: '`hostname` > /etc/mailhelo
ln -s /var/lib/mysql/mysql.err /var/lib/mysql/`hostname`.err

# Add /usr/bin/python link
ln -s /usr/bin/python2.7 /usr/bin/python

# Default Style Basic
# https://support.cpanel.net/hc/en-us/articles/1500011608461-How-to-remove-the-Switch-to-Glass-banner-from-the-cPanel-interface
whmapi1 set_default type='default' name='basic'
mkdir -pv /root/cpanel3-skel/.cpanel/nvdata/
echo -n "1" > /root/cpanel3-skel/.cpanel/nvdata/xmainNewStyleBannerDismissed

# 客戶用改成另外一組 CSF GLOBAL ALLOW & DENY
if [ "$host_use_user" == "2" ]; then
sed -i 's/^GLOBAL_ALLOW.*.=.*/GLOBAL_ALLOW = "https:\/\/data.yji.tw\/csf\/glob_allow_ma.rules"/g' /etc/csf/csf.conf
csf -ra
fi

# remove WordPress-Toolkit-Plugin
# https://support.cpanel.net/hc/en-us/articles/360058369013-How-can-I-remove-the-WordPress-Toolkit-Plugin-
rpm -e wp-toolkit-cpanel

# 如果是cloudlinuxOS，將加裝以下套件 (目前php81 json沒包含這兩個套件)
if [ "$CT" ]; then
  dnf install -y ea-libcurl ea-libcurl-devel
fi

# Add python link
ln -s /usr/bin/python2 /usr/bin/python

# 修正v106語系檔案消失問題
/opt/cpinst/jacky_script/import_cplocales.sh

# 修正ipv6優先權高於ipv4問題
touch /etc/gai.conf
cat << EOF >> /etc/gai.conf
label ::1/128 0
label ::/0 1
label 2002::/16 2
label ::/96 3
label ::ffff:0:0/96 4
precedence ::1/128 50
precedence ::/0 40
precedence 2002::/16 30
precedence ::/96 20
precedence ::ffff:0:0/96 100
EOF

# Convert Roundcube to SQLite database 
#/usr/local/cpanel/scripts/convert_roundcube_mysql2sqlite

# fix csf CSF MESSENGER bug
systemctl stop nftables
systemctl disable nftables
csf -ra

# CSF 一小時內寄送不存在信箱超過十次則封鎖IP
if [ "$host_use_user" == "1" ]; then
cat /opt/cpinst/cpinst/config/regex.custom.pm > /etc/csf/regex.custom.pm
sed -i 's/^CUSTOM1_LOG.=.*/CUSTOM1_LOG = "\/var\/log\/exim_mainlog"/g' /etc/csf/csf.conf
csf -ra
fi

# 再次執行安裝 PHP Selector & Lsapi (有時候 alt-php 套件會安裝失敗，作業流程結束前再安裝一次)
if [ "${virtualhost_type}" == "php" ] && [ "$installcagefs" == "yes" ]; then
   /bin/bash /opt/cpinst/cpinst/install_script/phpselector.sh
fi
if [ "$lsapi" == "yes" ]; then
/opt/cpinst/cpinst/install_script/lsapi.sh
fi


# 紀錄結束安裝時間
echo "結束安裝 : `date`" >> /root/install_date

# 說明已安裝完畢
echo ""
echo "cpinst8.sh 已執行安裝完畢，接著請依照wiki初始設定進行手動設定與確認。"
echo ""
echo "Zabbix 網頁頁面設定監控項目請參考 https://wiki.noc.tw/Zabbix_建立新主機"
echo ""

# Easyapache4 未正確套用指定PHP版本json檔案判斷

if [ ! -d "/etc/cpanel/ea4/profiles/custom/" ]; then

echo "#########################"
echo ""
echo "注意 !!! 此次 cPanel 安裝 Easyapache4 並未正確套用指定 PHP版本 josn 檔案，請手動執行以下指令"
echo ""
echo "/usr/local/bin/ea_install_profile --install /etc/cpanel_initial_install_ea4_profile.json"
echo ""
echo "執行完畢後，進入 WHM > Easyapache4 確認目前預設 PHP版本 是否為安裝指定的版本"
echo ""
echo "接著執行 /opt/cpinst/cpinst/install_script/lsapi.sh 指令重新套用lsapi設定"
echo ""
echo "最後確認 WHM > MultiPHP Manager 功能底下 PHP Handlers 全部版本是否都改成 lsapi"
echo ""
echo "如果遇到不可預期的問題，請回報讓二線與主管知道這件事情，後續才能排查問題點 !!!"
echo ""
echo "#########################"

fi

